@@ -1,12 +1,15 @@
-from typing import Optional, List, Dict, Any
+from typing import Optional, List, Dict, Any, Set
 from StructNoSQL.dynamodb.dynamodb_core import DynamoDbCoreAdapter, PrimaryIndex, GlobalSecondaryIndex, \
     DynamoDBMapObjectSetter, Response
-from StructNoSQL.dynamodb.models import DatabasePathElement, FieldGetter, FieldSetter, FieldRemover
+from StructNoSQL.dynamodb.models import DatabasePathElement, FieldGetter, FieldSetter, UnsafeFieldSetter, FieldRemover
 from StructNoSQL.fields import BaseField, MapModel, MapField, MapItem, TableDataModel
 from StructNoSQL.practical_logger import message_with_vars
 from StructNoSQL.utils.process_render_fields_paths import process_and_get_fields_paths_objects_from_fields_paths, \
-    process_and_make_single_rendered_database_path, process_validate_data_and_make_single_rendered_database_path
+    process_and_make_single_rendered_database_path, process_validate_data_and_make_single_rendered_database_path, \
+    process_and_get_field_path_object_from_field_path, make_rendered_database_path
 from StructNoSQL.utils.types import PRIMITIVE_TYPES
+from StructNoSQL.utils.decimals import float_to_decimal_serializer
+
 
 # todo: add ability to add or remove items from list's
 
@@ -34,6 +37,7 @@ class BaseTable:
             self._model = data_model()
         self._model_virtual_map_field = None
 
+        self.processed_class_types: Set[type] = set()
         assign_internal_mapping_from_class(table=self, class_instance=self._model)
 
     @property
@@ -117,15 +121,16 @@ class BaseTable:
         return response_data
 
 
-    def query(self, key_name: str, key_value: str, fields_to_get: List[str], index_name: Optional[str] = None,
-              limit: Optional[int] = None, query_kwargs: Optional[dict] = None) -> Optional[List[Any]]:
+    def query(self, key_name: str, key_value: str, fields_to_get: List[str], index_name: Optional[str] = None, limit: Optional[int] = None,
+              query_kwargs: Optional[dict] = None, filter_expression: Optional[Any] = None, **additional_kwargs) -> Optional[List[Any]]:
         # rendered_fields_paths = make_rendered_fields_paths(fields_paths=fields_to_get, query_kwargs=query_kwargs)
         fields_paths_objects = process_and_get_fields_paths_objects_from_fields_paths(
             fields_paths=fields_to_get, fields_switch=self.fields_switch
         )
         response = self.dynamodb_client.query_by_key(
-            key_name=key_name, key_value=key_value, fields_to_get=fields_to_get,
-            index_name=index_name, query_limit=limit
+            key_name=key_name, key_value=key_value,
+            fields_to_get=fields_to_get, index_name=index_name, query_limit=limit,
+            filter_expression=filter_expression, **additional_kwargs
         )
         if response is not None:
             for current_item in response.items:
@@ -154,16 +159,38 @@ class BaseTable:
             return True if response is not None else False
         return False
 
-    def set_update_multiple_fields(self, key_name: str, key_value: str, setters: List[FieldSetter]) -> bool:
+    def set_update_multiple_fields(self, key_name: str, key_value: str, setters: List[FieldSetter or UnsafeFieldSetter]) -> bool:
         dynamodb_setters: List[DynamoDBMapObjectSetter] = list()
         for current_setter in setters:
-            validated_data, valid, target_path_elements = process_validate_data_and_make_single_rendered_database_path(
-                field_path=current_setter.target_path, fields_switch=self.fields_switch,
-                query_kwargs=current_setter.query_kwargs, data_to_validate=current_setter.value_to_set
-            )
-            if valid is True:
+            if isinstance(current_setter, FieldSetter):
+                validated_data, valid, target_path_elements = process_validate_data_and_make_single_rendered_database_path(
+                    field_path=current_setter.target_path, fields_switch=self.fields_switch,
+                    query_kwargs=current_setter.query_kwargs, data_to_validate=current_setter.value_to_set
+                )
+                if valid is True:
+                    dynamodb_setters.append(DynamoDBMapObjectSetter(
+                        target_path_elements=target_path_elements, value_to_set=validated_data
+                    ))
+            elif isinstance(current_setter, UnsafeFieldSetter):
+                safe_target_field = process_and_get_field_path_object_from_field_path(
+                    field_path_key=current_setter.safe_base_target_path, fields_switch=self.fields_switch
+                )
+                if current_setter.unsafe_path_continuation is None:
+                    target_path_elements = safe_target_field.database_path
+                else:
+                    target_path_elements = safe_target_field.database_path + current_setter.unsafe_path_continuation
+
+                processed_value_to_set: Any = float_to_decimal_serializer(current_setter.value_to_set)
+                # Since the data is not validated, we pass it to the float_to_decimal_serializer
+                # function (which normally should be called by the data validation function)
+
+                rendered_target_path_elements = make_rendered_database_path(
+                    database_path_elements=target_path_elements,
+                    query_kwargs=current_setter.query_kwargs
+                )
                 dynamodb_setters.append(DynamoDBMapObjectSetter(
-                    target_path_elements=target_path_elements, value_to_set=validated_data
+                    target_path_elements=rendered_target_path_elements,
+                    value_to_set=processed_value_to_set
                 ))
 
         response = self.dynamodb_client.set_update_multiple_data_elements_to_map(
@@ -241,8 +268,26 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                         "to the assign_internal_mapping_from_class function"
             ))
 
+    if class_type in table.processed_class_types:
+        return None
+    else:
+        table.processed_class_types.update({class_type})
+
     class_variables = class_type.__dict__
     required_fields = list()
+    setup_function: Optional[callable] = class_variables.get('__setup__', None)
+    if setup_function is not None:
+        custom_setup_class_variables: dict = class_type.__setup__()
+        if len(custom_setup_class_variables) > 0:
+            # The class_variables gotten from calling the __dict__ attribute is a mappingproxy, which cannot be modify.
+            # In order to combine the custom_setup_class_variables and the class_variables variables we will iterate
+            # over all the class_variables attributes, add them to the dict create by the __setup__ function (only if
+            # they are not found in the custom_setup_class_variables dict, since the custom setup override any default
+            # class attribute), and assign the class_variables variable to our newly create and setup dict.
+            for key, item in class_variables.items():
+                if key not in custom_setup_class_variables:
+                    custom_setup_class_variables[key] = item
+            class_variables = custom_setup_class_variables
 
     for variable_key, variable_item in class_variables.items():
         current_field_path = "" if nested_field_path is None else f"{nested_field_path}"
@@ -250,6 +295,7 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
         try:
             if isinstance(variable_item, MapField):
                 variable_item: MapField
+
                 new_database_path_element = DatabasePathElement(
                     element_key=variable_item.field_name,
                     default_type=variable_item.field_type,
