@@ -4,7 +4,7 @@ from StructNoSQL.dynamodb.models import DatabasePathElement
 from StructNoSQL.fields import BaseItem, BaseField, MapModel
 from StructNoSQL.field_loader import load as field_load
 from StructNoSQL.practical_logger import message_with_vars
-from StructNoSQL.utils.decimals import float_to_decimal
+from StructNoSQL.utils.decimals import float_to_decimal, float_to_decimal_serializer
 
 NoneType = type(None)
 
@@ -17,6 +17,9 @@ def validate_data(value: Any, expected_value_type: Any, load_data_into_objects:
 
     value_type = type(value)
 
+    if expected_value_type == Any:
+        return float_to_decimal_serializer(value), True
+
     if type(expected_value_type) in [list, tuple]:
         has_found_match = False
         for acceptable_value_type in expected_value_type:
